@@ -1,9 +1,9 @@
-# -*- coding: utf-8 -*-
-from PyQt4.QtGui import *
-from PyQt4.QtCore import *
-from puddlestuff.constants import RIGHTDOCK
-import sys
-from puddlestuff.translations import translate
+from PyQt5.QtCore import QMutex
+from PyQt5.QtGui import QTextOption
+from PyQt5.QtWidgets import QApplication, QHBoxLayout, QPushButton, QTextEdit, QVBoxLayout, QWidget
+
+from ..constants import RIGHTDOCK
+from ..translations import translate
 
 mutex = QMutex()
 
@@ -20,8 +20,8 @@ class LogDialog(QWidget):
         copy = QPushButton(translate("Logs", '&Copy'))
         clear = QPushButton(translate("Logs", '&Clear'))
 
-        self.connect(copy, SIGNAL('clicked()'), self._copy)
-        self.connect(clear, SIGNAL('clicked()'), self._clear)
+        copy.clicked.connect(self._copy)
+        clear.clicked.connect(self._clear)
 
         vbox = QVBoxLayout()
         vbox.addWidget(self._text)
@@ -50,10 +50,11 @@ class LogDialog(QWidget):
         else:
             self._text.setPlaintext(text)
 
+
 control = ('Logs', LogDialog, RIGHTDOCK, False)
 
 if __name__ == '__main__':
     app = QApplication([])
     win = LogDialog()
     win.show()
-    app.exec_()
\ No newline at end of file
+    app.exec_()
