@@ -1,81 +1,80 @@
-# -*- coding: utf-8 -*-
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-import pdb, sys, os, sip
-from puddlestuff.audioinfo import GENRES, INFOTAGS, READONLY
-from puddlestuff.audioinfo.util import commonimages
-from puddlestuff.puddleobjects import ListButtons, PuddleConfig, PicWidget
-import puddlestuff.resource as resource
+import os
+import sip
+import sys
+
+from PyQt5.QtCore import QEvent, QThread, Qt, pyqtRemoveInputHook, pyqtSignal
+from PyQt5.QtGui import QBrush
+from PyQt5.QtWidgets import QApplication, QComboBox, QCompleter, QDialog, QGroupBox, QHBoxLayout, \
+    QLabel, QLineEdit, QTableWidget, QTableWidgetItem, QVBoxLayout, QWidget
+
+from ..audioinfo import INFOTAGS
+from ..puddleobjects import ListButtons, PuddleConfig
+
 pyqtRemoveInputHook()
-from puddlestuff.constants import LEFTDOCK, SELECTIONCHANGED, BLANK, KEEP, SEPARATOR
+from ..constants import LEFTDOCK, SELECTIONCHANGED, BLANK, KEEP, SEPARATOR
 from functools import partial
-try:
-    from puddlestuff.puddlesettings import SettingsError
-except ImportError:
-    pass
-from puddlestuff.translations import translate
-from puddlestuff.constants import CONFIGDIR
+from ..puddlesettings import SettingsError
+from ..translations import translate
+from ..constants import CONFIGDIR
 
-TEXTEDITED = SIGNAL('textEdited(const QString&)')
-EDITFINISHED = SIGNAL('editingFinished()')
-INDEXCHANGED = SIGNAL('currentIndexChanged(const QString&)')
 
-def loadsettings(filepath = None):
+def loadsettings(filepath=None):
     settings = PuddleConfig()
     if filepath:
         settings.filename = filepath
     else:
         settings.filename = os.path.join(CONFIGDIR, 'tagpanel')
-    numrows = settings.get('panel','numrows',-1, True)
+    numrows = settings.get('panel', 'numrows', -1, True)
     if numrows > -1:
         sections = settings.sections()
         d = {}
-        for row in xrange(numrows):
-            section = unicode(row)
+        for row in range(numrows):
+            section = str(row)
             tags = settings.get(section, 'tags', [''])
             titles = settings.get(section, 'titles', [''])
-            d[row] = zip(titles, tags)
+            d[row] = list(zip(titles, tags))
     else:
-        titles = ['&Artist', '&Title', 'Al&bum', 'T&rack', u'&Year', "&Genre", '&Comment']
-        tags = ['artist', 'title', 'album', 'track', u'year', 'genre', 'comment']
-        newtags = zip(titles, tags)
-        d = {0:[newtags[0]], 1:[newtags[1]], 2: [newtags[2]],
-             3:[newtags[3], newtags[4], newtags[5]] ,
-             4:[newtags[6]]}
+        titles = ['&Artist', '&Title', 'Al&bum', 'T&rack', '&Year', "&Genre", '&Comment']
+        tags = ['artist', 'title', 'album', 'track', 'year', 'genre', 'comment']
+        newtags = list(zip(titles, tags))
+        d = {0: [newtags[0]], 1: [newtags[1]], 2: [newtags[2]],
+             3: [newtags[3], newtags[4], newtags[5]],
+             4: [newtags[6]]}
     return d
 
+
 def savesettings(d, filepath=None):
     settings = PuddleConfig()
     if filepath:
         settings.filename = filepath
     else:
         settings.filename = os.path.join(settings.savedir, 'tagpanel')
-    settings.set('panel', 'numrows', unicode(len(d)))
+    settings.set('panel', 'numrows', str(len(d)))
     for row, rowtags in d.items():
-        settings.set(unicode(row), 'tags', [z[1] for z in rowtags])
-        settings.set(unicode(row), 'titles', [z[0] for z in rowtags])
+        settings.set(str(row), 'tags', [z[1] for z in rowtags])
+        settings.set(str(row), 'titles', [z[0] for z in rowtags])
+
 
 class Combo(QComboBox):
 
     def __init__(self, *args, **kwargs):
         super(Combo, self).__init__(*args, **kwargs)
         self._edited = False
+
         def k(text): self._edited = True
-        self.connect(self, SIGNAL('editTextChanged(const QString&)'),
-            k)        
+
+        self.editTextChanged.connect(k)
 
     def setEditText(self, text):
-        print 'setText'
         self._edited = False
         super(Combo, self).setEditText(text)
-    
+
     def focusOutEvent(self, event):
-        print self._edited
         if not self._edited:
             return super(Combo, self).focusOutEvent(event)
         curtext = self.currentText()
         index = self.findText(curtext,
-            Qt.MatchExactly | Qt.MatchFixedString | Qt.MatchCaseSensitive)
+                              Qt.MatchExactly | Qt.MatchFixedString | Qt.MatchCaseSensitive)
         if index == -1:
             index = self.findText(curtext, Qt.MatchExactly | Qt.MatchFixedString)
         if index > 1:
@@ -85,13 +84,17 @@ class Combo(QComboBox):
         self._edited = False
         return super(Combo, self).focusOutEvent(event)
 
+
 class Thread(QThread):
+    update = pyqtSignal(object, name='update')
+
     def __init__(self, values, parent=None):
         self._values = values
         QThread.__init__(self, parent)
 
     def run(self):
-        self.emit(SIGNAL('update'), self._values)
+        self.update.emit(self._values)
+
 
 class FrameCombo(QGroupBox):
     """A group box with combos that allow to edit
@@ -106,23 +109,28 @@ class FrameCombo(QGroupBox):
     which is a dictionary key = tag, value = respective combobox.
     """
 
-    def __init__(self,tags = None,parent= None, status=None):
+    onetomany = pyqtSignal(dict, name='onetomany')
+    onetomanypreview = pyqtSignal(dict, name='onetomanypreview')
+    manypreview = pyqtSignal(list, name='manypreview')
+
+    def __init__(self, tags=None, parent=None, status=None):
         self.settingsdialog = SettingsWin
-        QGroupBox.__init__(self,parent)
+        QGroupBox.__init__(self, parent)
         self.emits = ['onetomany', 'onetomanypreview', 'manypreview']
         self.receives = [(SELECTIONCHANGED, self.fillCombos),
-            ('previewModeChanged', lambda v: self._enablePreview()
-                if v else self._disablePreview()),]
+                         ('previewModeChanged', lambda v: self._enablePreview()
+                         if v else self._disablePreview()), ]
         self.combos = {}
         self.labels = {}
+        self._hboxes = []
         self._status = status
         self._originalValues = {}
         self.__indexFuncs = []
-    
+
     def _disablePreview(self):
         self.__disconnectIndexChanged
         self.__indexFuncs = []
-        for field, combo in self.combos.iteritems():
+        for field, combo in self.combos.items():
             edit = QLineEdit()
             combo.setLineEdit(edit)
             completer = combo.completer()
@@ -133,38 +141,42 @@ class FrameCombo(QGroupBox):
         for z in self.combos:
             self.combos[z].clear()
             self.combos[z].setEnabled(False)
-    
+
     def _enablePreview(self):
         self.__indexFuncs = []
-        for field, combo in self.combos.iteritems():
+        for field, combo in self.combos.items():
             func = partial(self._emitChange, field)
             edit = QLineEdit()
             combo.setLineEdit(edit)
             completer = combo.completer()
             completer.setCaseSensitivity(Qt.CaseSensitive)
             completer.setCompletionMode(QCompleter.UnfilteredPopupCompletion)
-            self.connect(edit, TEXTEDITED, func)
-            combo.connect(combo, INDEXCHANGED, func)
+            edit.textEdited.connect(func)
+            combo.currentIndexChanged.connect(func)
             self.__indexFuncs.append((combo, func))
-            self.connect(edit, EDITFINISHED, self.save)
+            edit.editingFinished.connect(self.save)
 
     def __disconnectIndexChanged(self):
         for combo, func in self.__indexFuncs:
             combo.disconnect(combo, func)
-    
+
     def _emitChange(self, field, text):
-        text = unicode(text)
-        if text == BLANK: text = u''
+        if type(text) is int:
+            text = self.combos[field].itemText(text)
+        else:
+            text = str(text)
+
+        if text == BLANK:
+            text = ''
         elif text == KEEP:
             if self._audios:
-                self.emit(SIGNAL('manypreview'),
-                    [{field: a.get(field, u'')} for a in self._audios])
+                self.manypreview.emit([{field: a.get(field, '')} for a in self._audios])
         else:
             if field in INFOTAGS:
                 value = text
             else:
                 value = text.split(SEPARATOR)
-        self.emit(SIGNAL('onetomanypreview'), {field: text})
+        self.onetomanypreview.emit({field: text})
 
     def fillCombos(self, *args):
         audios = self._status['selectedfiles']
@@ -184,18 +196,20 @@ class FrameCombo(QGroupBox):
             for field in tags:
                 if field in audio:
                     value = audio[field]
-                    if isinstance(value, basestring):
+                    if isinstance(value, str):
                         tags[field].add(value)
                     else:
                         tags[field].add(SEPARATOR.join(value))
                 else:
-                    tags[field].add(u'')
+                    tags[field].add('')
 
-        for field, values in tags.iteritems():
+        for field, values in tags.items():
             combo = combos[field]
             combo.addItems(sorted(values))
-            if len(values) == 1: combo.setCurrentIndex(2)
-            else: combo.setCurrentIndex(0)
+            if len(values) == 1:
+                combo.setCurrentIndex(2)
+            else:
+                combo.setCurrentIndex(0)
 
         if 'genre' in tags and len(tags['genre']) == 1:
             combo = combos['genre']
@@ -208,8 +222,8 @@ class FrameCombo(QGroupBox):
         elif 'genre' in tags:
             combos['genre'].setCurrentIndex(0)
 
-        self._originalValues = dict([(field, unicode(combo.currentText()))
-            for field, combo in self.combos.items()])
+        self._originalValues = dict([(field, str(combo.currentText()))
+                                     for field, combo in self.combos.items()])
         self._originalValues['__image'] = self._status['images']
         [combo.blockSignals(False) for combo in combos.values()]
 
@@ -223,12 +237,14 @@ class FrameCombo(QGroupBox):
             tags['__image'] = images
         originals = {}
         for field, combo in combos.items():
-            curtext = unicode(combo.currentText())
+            curtext = str(combo.currentText())
             if self._originalValues[field] == curtext:
                 continue
             originals[field] = curtext
-            if curtext == BLANK: tags[field] = []
-            elif curtext == KEEP: continue
+            if curtext == BLANK:
+                tags[field] = []
+            elif curtext == KEEP:
+                continue
             else:
                 if field in INFOTAGS:
                     tags[field] = curtext
@@ -237,7 +253,7 @@ class FrameCombo(QGroupBox):
 
         if '__image' in tags:
             if self._originalValues['__image'] == tags['__image']:
-                del(tags['__image'])
+                del (tags['__image'])
             else:
                 originals['__image'] = tags['__image']
 
@@ -245,15 +261,15 @@ class FrameCombo(QGroupBox):
             return
 
         self._originalValues.update(originals)
-        self.emit(SIGNAL('onetomany'), tags)
+        self.onetomany.emit(tags)
         if 'genre' in combos:
             combo = combos['genre']
-            
-            genres = self._status['genres']
-            new_genres = filter(None, unicode(combo.currentText()).split(SEPARATOR))
 
-            [genres.append(genre) for genre in new_genres 
-                if genre not in genres]
+            genres = self._status['genres']
+            new_genres = [_f for _f in str(combo.currentText()).split(SEPARATOR) if _f]
+
+            [genres.append(genre) for genre in new_genres
+             if genre not in genres]
 
     def setCombos(self, rowtags):
         """Creates a vertical column of comboboxes.
@@ -266,11 +282,11 @@ class FrameCombo(QGroupBox):
         row being before the comments row. You'd do something like...
 
         >>>tags = [('Artist', 'artist'), ('Title', 'title'), ('Album', 'album'),
-        ...        ('Track', 'track'), ("Comments",'comment'), ('Genre', 'genre'), (u'Year', u'date')]
+        ...        ('Track', 'track'), ("Comments",'comment'), ('Genre', 'genre'), ('Year', 'date')]
         >>>rows = {0:[0], 1:[1], 2:[2], 3[3,4,6],4:[5]
         >>>f = FrameCombo()
         >>>f.setCombo(tags, rows)"""
-        #pdb.set_trace()
+        # pdb.set_trace()
 
         if self.combos:
             vbox = self.layout()
@@ -282,7 +298,7 @@ class FrameCombo(QGroupBox):
                     sip.delete(box)
         else:
             vbox = QVBoxLayout()
-            vbox.setMargin(0)
+            vbox.setContentsMargins(0, 0, 0, 0)
         self.combos = {}
         self.labels = {}
         self._hboxes = []
@@ -292,7 +308,7 @@ class FrameCombo(QGroupBox):
             labelbox = QHBoxLayout()
             labelbox.setContentsMargins(6, 1, 1, 1)
             widgetbox = QHBoxLayout()
-            widgetbox.setMargin(0)
+            widgetbox.setContentsMargins(0, 0, 0, 0)
             for tag in tags:
                 tagval = tag[1]
                 self.labels[tagval] = QLabel(tag[0])
@@ -310,7 +326,7 @@ class FrameCombo(QGroupBox):
                 self._hboxes.append((widgetbox, self.combos[tagval]))
             vbox.addLayout(labelbox)
             vbox.addLayout(widgetbox)
-        
+
         self.setLayout(vbox)
         QApplication.processEvents()
         self.setMaximumHeight(self.sizeHint().height())
@@ -339,6 +355,7 @@ class FrameCombo(QGroupBox):
     def loadSettings(self):
         self.setCombos(loadsettings())
 
+
 class PuddleTable(QTableWidget):
     def __init__(self, columns=None, defaultvals=None, parent=None):
         QTableWidget.__init__(self, parent)
@@ -349,7 +366,7 @@ class PuddleTable(QTableWidget):
             self.setHorizontalHeaderLabels(columns)
             self.horizontalHeader().setStretchLastSection(True)
 
-    def add(self, texts = None):
+    def add(self, texts=None):
         row = self.rowCount()
         item = self.horizontalHeaderItem
         if self._default and (not texts):
@@ -367,7 +384,7 @@ class PuddleTable(QTableWidget):
         for i in range(len(rows)):
             self.removeRow(rows[i])
             rows = [z - 1 for z in rows]
-        self.emit(SIGNAL('itemSelectionChanged()'))
+        self.itemSelectionChanged.emit()
 
     def moveUp(self):
         row = self.currentRow()
@@ -377,7 +394,7 @@ class PuddleTable(QTableWidget):
         newitems = self.texts(row)
         self.setRowTexts(row, previtems)
         self.setRowTexts(row - 1, newitems)
-        curitem = self.item(row-1, self.currentItem().column())
+        curitem = self.item(row - 1, self.currentItem().column())
         self.setCurrentItem(curitem)
 
     def moveDown(self):
@@ -388,30 +405,31 @@ class PuddleTable(QTableWidget):
         newitems = self.texts(row)
         self.setRowTexts(row, previtems)
         self.setRowTexts(row + 1, newitems)
-        curitem = self.item(row+1, self.currentItem().column())
+        curitem = self.item(row + 1, self.currentItem().column())
         self.setCurrentItem(curitem)
 
-    rows = property(lambda x: xrange(x.rowCount()))
+    rows = property(lambda x: range(x.rowCount()))
 
     def setRowTexts(self, row, texts):
         item = self.item
-        for column, text in zip(range(self.columnCount()), texts):
+        for column, text in zip(list(range(self.columnCount())), texts):
             item(row, column).setText(text)
 
     def texts(self, row):
         item = self.item
-        return [unicode(item(row, z).text()) for z in range(self.columnCount())]
+        return [str(item(row, z).text()) for z in range(self.columnCount())]
 
     def items(self, row):
         item = self.item
         return [item(row, z) for z in range(self.columnCount())]
 
     def text(self, row, column):
-        return unicode(self.item(row, column).text())
+        return str(self.item(row, column).text())
 
     def selectedRows(self):
         return sorted(set(i.row() for i in self.selectedIndexes()))
 
+
 TABLEWIDGETBG = QTableWidgetItem().background()
 RED = QBrush(Qt.red)
 
@@ -419,38 +437,39 @@ TITLE = translate("Defaults", 'Title')
 FIELD = translate("Defaults", 'Field')
 ROW = translate("Tag Panel Settings", 'Row')
 
+
 class SettingsWin(QWidget):
-    
-    def __init__(self, parent = None, status=None):
+
+    def __init__(self, parent=None, status=None):
         QDialog.__init__(self, parent)
         self.title = translate('Settings', 'Tag Panel')
         self._table = PuddleTable([TITLE, FIELD, ROW],
-            [TITLE, FIELD, u'0'], self)
+                                  [TITLE, FIELD, '0'], self)
         self._buttons = ListButtons()
         self._buttons.connectToWidget(self._table, add=self.add,
-            edit=self.edit, moveup=self._table.moveUp,
-            movedown=self._table.moveDown, duplicate=self.duplicate)
+                                      edit=self.edit, moveup=self._table.moveUp,
+                                      movedown=self._table.moveDown, duplicate=self.duplicate)
 
         hbox = QHBoxLayout()
         hbox.addWidget(self._table, 1)
         hbox.addLayout(self._buttons, 0)
         self.setLayout(hbox)
 
-        self.connect(self._table, SIGNAL('cellChanged(int,int)'),
-            self._checkItem)
-        self.connect(self._table, SIGNAL('itemSelectionChanged()'),
-            self._enableButtons)
+        self._table.cellChanged.connect(self._checkItem)
+        self._table.itemSelectionChanged.connect(self._enableButtons)
         self.fill()
 
-    def add(self, texts = None):
+    def add(self, texts=None):
         table = self._table
         if not texts:
             text = table.text
             rows = []
             for row in table.rows:
-                try: rows.append(int(text(row, 2)))
-                except (TypeError, ValueError): pass
-            row = unicode(max(rows) + 1) if rows else u'1'
+                try:
+                    rows.append(int(text(row, 2)))
+                except (TypeError, ValueError):
+                    pass
+            row = str(max(rows) + 1) if rows else '1'
             table.add([TITLE, FIELD.lower(), row])
         else:
             table.add(texts)
@@ -463,9 +482,9 @@ class SettingsWin(QWidget):
         self._old = d
         for row in d:
             for z in d[row]:
-                self._table.add(z + (unicode(row),))
+                self._table.add(z + (str(row),))
 
-    def applySettings(self, control = None):
+    def applySettings(self, control=None):
         texts = self._table.texts
         d = {}
         for row in self._table.rows:
@@ -474,12 +493,12 @@ class SettingsWin(QWidget):
                 l[2] = int(l[2])
             except ValueError:
                 raise SettingsError(translate('Tag Panel Settings',
-                'All row numbers must be integers.'))
+                                              'All row numbers must be integers.'))
             try:
                 d[l[2]].append(l[:-1])
             except KeyError:
                 d[l[2]] = [l[:-1]]
-        d = dict([(i, d[v]) for i, v in enumerate(sorted(d))]) #consecutive rows
+        d = dict([(i, d[v]) for i, v in enumerate(sorted(d))])  # consecutive rows
         if self._old == d:
             return
         savesettings(d)
@@ -492,8 +511,8 @@ class SettingsWin(QWidget):
         table = self._table
         if column == 2:
             try:
-               int(table.text(row, column))
-               table.item(row, column).setBackground(TABLEWIDGETBG)
+                int(table.text(row, column))
+                table.item(row, column).setBackground(TABLEWIDGETBG)
             except ValueError:
                 i = table.item(row, column)
                 i.setBackground(RED)
@@ -501,13 +520,13 @@ class SettingsWin(QWidget):
     def _enableButtons(self):
         table = self._table
         if table.rowCount() <= 0:
-            self._buttons.edit.setEnabled(False)
-            self._buttons.duplicate.setEnabled(False)
-            self._buttons.remove.setEnabled(False)
+            self._buttons.editButton.setEnabled(False)
+            self._buttons.duplicateButton.setEnabled(False)
+            self._buttons.removeButton.setEnabled(False)
         elif self._table.selectedRows():
-            self._buttons.edit.setEnabled(True)
-            self._buttons.duplicate.setEnabled(True)
-            self._buttons.remove.setEnabled(True)
+            self._buttons.editButton.setEnabled(True)
+            self._buttons.duplicateButton.setEnabled(True)
+            self._buttons.removeButton.setEnabled(True)
 
     def duplicate(self):
         table = self._table
@@ -522,4 +541,4 @@ if __name__ == '__main__':
     app = QApplication(sys.argv)
     win = SettingsWin()
     win.show()
-    app.exec_()
\ No newline at end of file
+    app.exec_()
