@@ -1,28 +1,28 @@
-#! /usr/bin/env python
-# -*- coding: utf-8 -*-
-import glob, os, pdb, sys, traceback
+import glob
+import logging
+import os
+import sys
+import traceback
+from copy import deepcopy
 
-from collections import defaultdict
-from copy import copy, deepcopy
-from functools import partial
+from PyQt5.QtCore import Qt, pyqtRemoveInputHook, pyqtSignal
+from PyQt5.QtGui import QIcon
+from PyQt5.QtWidgets import QApplication, QCheckBox, QComboBox, QDialog, QGroupBox, QHBoxLayout, \
+    QInputDialog, QLabel, QLineEdit, QPushButton, QSpinBox, QTextEdit, QToolButton, QVBoxLayout, \
+    QWidget
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-
-import puddlestuff
-
-from puddlestuff.releasewidget import ReleaseWidget
-import puddlestuff.audioinfo as audioinfo
-from puddlestuff.constants import (TEXT, COMBO, SPINBOX,
-                                   CHECKBOX, RIGHTDOCK, CONFIGDIR)
-from puddlestuff.findfunc import parsefunc
-from puddlestuff.functions import replace_regex
-from puddleobjects import (create_buddy, unique, winsettings,
-    ListBox, ListButtons, OKCancel, PuddleConfig, PuddleThread)
-from puddlestuff.tagsources import (tagsources, status_obj, set_useragent,
-    write_log, RetrievalError, mp3tag, SubmissionError)
-from puddlestuff.util import (isempty, pprint_tag,
-    split_by_field, to_string, translate)
+from . import audioinfo, version_string
+from .constants import (TEXT, COMBO, SPINBOX,
+                        CHECKBOX, RIGHTDOCK, CONFIGDIR)
+from .findfunc import FuncError
+from .functions import replace_regex
+from .puddleobjects import (create_buddy, winsettings,
+                            ListBox, ListButtons, OKCancel, PuddleConfig, PuddleThread)
+from .releasewidget import ReleaseWidget
+from .tagsources import (tagsources, status_obj, set_useragent,
+                         write_log, RetrievalError, mp3tag, SubmissionError)
+from .util import (isempty, pprint_tag,
+                   split_by_field, to_string, translate)
 
 pyqtRemoveInputHook()
 
@@ -30,32 +30,33 @@ TAGSOURCE_CONFIG = os.path.join(CONFIGDIR, 'tagsources.conf')
 MTAG_SOURCE_DIR = os.path.join(CONFIGDIR, 'mp3tag_sources')
 
 DEFAULT_SEARCH_TIP = translate("WebDB",
-    "Enter search parameters here. If empty, the selected "
-    "files are used. <ul><li><b>artist;album</b> searches "
-    "for a specific album/artist combination.</li><li>To "
-    "list the albums by an artist leave off the album "
-    "part, but keep the semicolon (eg. <b>Ratatat;</b>). "
-    "For a album only leave the artist part as in "
-    "<b>;Resurrection.</li></ul>")
+                               "Enter search parameters here. If empty, the selected "
+                               "files are used. <ul><li><b>artist;album</b> searches "
+                               "for a specific album/artist combination.</li><li>To "
+                               "list the albums by an artist leave off the album "
+                               "part, but keep the semicolon (eg. <b>Ratatat;</b>). "
+                               "For a album only leave the artist part as in "
+                               "<b>;Resurrection.</li></ul>")
 
 FIELDLIST_TIP = translate("WebDB",
-    'Enter a comma seperated list of fields to write. '
-    '<br /><br />Eg. <b>artist, album, title</b> will only '
-    'write the artist, album and title fields of the '
-    'retrieved tags. <br /><br />If you want to '
-    'exclude some fields, but write all others start the '
-    'list the tilde (~) character. Eg <b>~composer, '
-    '__image</b> will write all fields but the '
-    'composer and __image fields.')
+                          'Enter a comma seperated list of fields to write. '
+                          '<br /><br />Eg. <b>artist, album, title</b> will only '
+                          'write the artist, album and title fields of the '
+                          'retrieved tags. <br /><br />If you want to '
+                          'exclude some fields, but write all others start the '
+                          'list the tilde (~) character. Eg <b>~composer, '
+                          '__image</b> will write all fields but the '
+                          'composer and __image fields.')
+
+DEFAULT_REGEXP = {'album': [u'(.*?)([\(\[\{].*[\)\]\}])', '$1']}
 
-DEFAULT_REGEXP = {'album': [u'(.*?)([\(\[\{].*[\)\]\}])', u'$1']}
 
 def apply_regexps(audio, regexps=None):
     if regexps is None:
         regexps = DEFAULT_REGEXP
     audio = deepcopy(audio)
     changed = False
-    for field, (regexp, output) in regexps.iteritems():
+    for field, (regexp, output) in regexps.items():
         if field not in audio:
             continue
         text = to_string(audio[field])
@@ -65,23 +66,25 @@ def apply_regexps(audio, regexps=None):
                 audio[field] = val
                 if not changed and val != text:
                     changed = val
-        except puddlestuff.findfunc.FuncError:
+        except FuncError:
             continue
     return changed, audio
 
+
 def display_tag(tag):
     """Used to display tags in in a human parseable format."""
-    tag = dict((k,v) for k,v in tag.iteritems() if
-        not k.startswith('#') and not isempty(v))
+    tag = dict((k, v) for k, v in tag.items() if
+               not k.startswith('#') and not isempty(v))
 
     if not tag:
         return translate("WebDB", "<b>Nothing to display.</b>")
-    fmt = u"<b>%s</b>: %s<br />"
+    fmt = "<b>%s</b>: %s<br />"
     text = pprint_tag(tag, fmt, True)
-    if text.endswith(u'<br />'):
-        text = text[:-len(u'<br />')]
+    if text.endswith('<br />'):
+        text = text[:-len('<br />')]
     return text
 
+
 def load_mp3tag_sources(dirpath=MTAG_SOURCE_DIR):
     "Loads Mp3tag tag sources from dirpath and return the tag source classes."
     files = glob.glob(os.path.join(dirpath, '*.src'))
@@ -91,12 +94,12 @@ def load_mp3tag_sources(dirpath=MTAG_SOURCE_DIR):
             idents, search, album = mp3tag.open_script(f)
             classes.append(mp3tag.Mp3TagSource(idents, search, album))
         except:
-            print translate("WebDB", "Couldn't load Mp3tag Tag Source %s") % f
-            traceback.print_exc()
+            logging.exception(translate("WebDB", "Couldn't load Mp3tag Tag Source %s") % f)
             continue
     return classes
 
-def strip(audio, field_list, reverse = False, leave_exact=False):
+
+def strip(audio, field_list, reverse=False, leave_exact=False):
     '''Returns dict of key/values from audio where the key is in field_list.
 
     If reverse is True then the dict will consist of all
@@ -109,7 +112,7 @@ def strip(audio, field_list, reverse = False, leave_exact=False):
     '''
     if not field_list:
         ret = dict([(key, audio[key]) for key in audio if
-            not key.startswith('#')])
+                    not key.startswith('#')])
         if leave_exact and '#exact' in audio:
             ret['#exact'] = audio['#exact']
         return ret
@@ -121,64 +124,70 @@ def strip(audio, field_list, reverse = False, leave_exact=False):
         reverse = False
     if reverse:
         ret = dict([(key, audio[key]) for key in audio if key not in
-                        tags and not key.startswith('#')])
+                    tags and not key.startswith('#')])
     else:
         ret = dict([(key, audio[key]) for key in field_list
-            if not key.startswith('#') and key in audio])
+                    if not key.startswith('#') and key in audio])
 
     if leave_exact and '#exact' in audio:
         ret['#exact'] = audio['#exact']
     return ret
 
+
 def split_strip(stringlist):
     '''Splits and strips each comma-delimited string in a list of strings.
 
     >>> split_strip(['artist, title', 'album,genre'])
     [['artist', 'title'], ['album', 'genre']]
     '''
-    return [[field.strip() for field in s.split(u',')] for s in stringlist]
+    return [[field.strip() for field in s.split(',')] for s in stringlist]
+
 
 class FieldsEdit(QWidget):
+    fieldsChanged = pyqtSignal(list, name='fieldsChanged')
+
     def __init__(self, tags=None, parent=None):
         QWidget.__init__(self, parent)
         if not tags:
             tags = []
         label = QLabel()
         label.setText(translate("Defaults", '&Fields'))
-        self._text = QLineEdit(u', '.join(tags))
+        self._text = QLineEdit(', '.join(tags))
         label.setBuddy(self._text)
 
         layout = QHBoxLayout()
-        layout.setMargin(0)
+        layout.setContentsMargins(0, 0, 0, 0)
         layout.addWidget(label, 0)
         layout.addWidget(self._text, 1)
 
-        self.connect(self._text, SIGNAL('textChanged(QString)'),
+        self._text.textChanged.connect(
             self.emitTags)
 
         self.setLayout(layout)
 
     def emitTags(self, text=None):
-        self.emit(SIGNAL('fieldsChanged'), self.tags(text))
+        self.fieldsChanged.emit(self.tags(text))
 
     def setTags(self, tags):
-        self._text.setText(u', '.join(tags))
-        
+        self._text.setText(', '.join(tags))
+
     def setToolTip(self, value):
         QWidget.setToolTip(self, value)
         self._text.setToolTip(value)
 
     def tags(self, text=None):
         if not text:
-            return filter(None, [z.strip() for z in
-                unicode(self._text.text()).split(u',')])
+            return [_f for _f in [z.strip() for z in
+                                  str(self._text.text()).split(',')] if _f]
         else:
-            return filter(None,
-                [z.strip() for z in unicode(text).split(u',')])
+            return [_f for _f in [z.strip() for z in str(text).split(',')] if _f]
+
 
 class SimpleDialog(QDialog):
     """Class for simple dialog creation."""
-    def __init__(self, title, controls, parent = None):
+    editingFinished = pyqtSignal(list, name='editingFinished')
+
+    def __init__(self, title, controls, parent=None):
         """title => Dialog's title.
         controls is a list of 3-element-lists.
 
@@ -237,11 +246,11 @@ class SimpleDialog(QDialog):
                 control.setMaximum(default[1])
                 control.setValue(default[2])
                 vbox.addLayout(create_buddy(desc, control))
-                
+
             self._controls.append(control)
         okcancel = OKCancel()
-        self.connect(okcancel, SIGNAL('ok'), self.okClicked)
-        self.connect(okcancel, SIGNAL('cancel'), self.close)
+        okcancel.ok.connect(self.okClicked)
+        okcancel.cancel.connect(self.close)
         vbox.addLayout(okcancel)
         vbox.addStretch()
         self.setLayout(vbox)
@@ -250,25 +259,28 @@ class SimpleDialog(QDialog):
         values = []
         for control in self._controls:
             if isinstance(control, QLineEdit):
-                values.append(unicode(control.text()))
+                values.append(str(control.text()))
             elif isinstance(control, QComboBox):
                 values.append(control.currentIndex())
             elif isinstance(control, QCheckBox):
                 values.append(control.isChecked())
             elif isinstance(control, QSpinBox):
                 values.append(control.value())
-        self.emit(SIGNAL('editingFinished'), values)
+        self.editingFinished.emit(values)
         self.close()
 
+
 class SortOptionEditor(QDialog):
-    def __init__(self, options, parent = None):
+    options = pyqtSignal(list, name='options')
+
+    def __init__(self, options, parent=None):
         """options is a list of strings. Each a comma-delimited field list.
 
         Eg. ['artist, title', 'album, genre']
         """
 
         QDialog.__init__(self, parent)
-        connect = lambda c, signal, s: self.connect(c, SIGNAL(signal), s)
+        connect = lambda c, signal, s: getattr(c, signal).connect(s)
         self.listbox = ListBox()
         self.listbox.setSelectionMode(self.listbox.ExtendedSelection)
 
@@ -279,7 +291,7 @@ class SortOptionEditor(QDialog):
         hbox.addWidget(self.listbox, 1)
 
         hbox.addLayout(buttons)
-        
+
         okcancel = OKCancel()
         vbox = QVBoxLayout()
         vbox.addLayout(hbox)
@@ -288,25 +300,24 @@ class SortOptionEditor(QDialog):
 
         connect(buttons, "add", self.addPattern)
         connect(buttons, "edit", self.editItem)
-        buttons.duplicate.setVisible(False)
-        self.connect(okcancel, SIGNAL('ok'), self.applySettings)
-        self.connect(okcancel, SIGNAL('cancel'), self.applySettings)
+        buttons.duplicateButton.setVisible(False)
+        okcancel.ok.connect(self.applySettings)
+        okcancel.cancel.connect(self.applySettings)
         self.listbox.connectToListButtons(buttons)
-        self.listbox.editButton = buttons.edit
-        connect(self.listbox, 'itemDoubleClicked(QListWidgetItem *)',
-                    self._doubleClicked)
+        self.listbox.editButton = buttons.editButton
+        connect(self.listbox, "itemDoubleClicked", self._doubleClicked)
 
     def addPattern(self):
         l = self.listbox.item
-        patterns = [unicode(l(z).text()) for z in range(self.listbox.count())]
+        patterns = [str(l(z).text()) for z in range(self.listbox.count())]
         row = self.listbox.currentRow()
         if row < 0:
             row = 0
         (text, ok) = QInputDialog().getItem(self, translate("WebDB",
-            'Add sort option'),
-            translate("WebDB",
-                'Enter a sorting option (a comma-separated list of fields. '
-                'Eg. "artist, title")'), patterns, row)
+                                                            'Add sort option'),
+                                            translate("WebDB",
+                                                      'Enter a sorting option (a comma-separated list of fields. '
+                                                      'Eg. "artist, title")'), patterns, row)
         if ok:
             self.listbox.clearSelection()
             self.listbox.addItem(text)
@@ -319,87 +330,88 @@ class SortOptionEditor(QDialog):
         if row is None:
             row = self.listbox.currentRow()
         l = self.listbox.item
-        patterns = [unicode(l(z).text()) for z in range(self.listbox.count())]
+        patterns = [str(l(z).text()) for z in range(self.listbox.count())]
         (text, ok) = QInputDialog().getItem(self,
-            translate("WebDB", 'Edit sort option'),
-            translate("WebDB",
-                'Enter a sorting option (a comma-separated list of fields. '
-                'Eg. "artist, title")'), patterns, row)
+                                            translate("WebDB", 'Edit sort option'),
+                                            translate("WebDB",
+                                                      'Enter a sorting option (a comma-separated list of fields. '
+                                                      'Eg. "artist, title")'), patterns, row)
         if ok:
             item = l(row)
             item.setText(text)
-            self.listbox.setItemSelected(item, True)
+            item.setSelected(True)
 
     def applySettings(self):
         item = self.listbox.item
-        options = [unicode(item(row).text())
-            for row in xrange(self.listbox.count())]
+        options = [str(item(row).text())
+                   for row in range(self.listbox.count())]
         self.close()
-        self.emit(SIGNAL('options'), options)
+        self.options.emit(options)
+
 
 class SettingsDialog(QWidget):
-    def __init__(self, parent = None, status = None):
+    def __init__(self, parent=None, status=None):
         QWidget.__init__(self, parent)
         self.title = translate('Settings', 'Tag Sources')
 
         label = QLabel(translate("WebDB",
-            '&Display format for individual tracks.'))
+                                 '&Display format for individual tracks.'))
         self._text = QLineEdit()
         label.setBuddy(self._text)
-        
+
         albumlabel = QLabel(translate("WebDB",
-            'Display format for &retrieved albums'))
+                                      'Display format for &retrieved albums'))
         self._albumdisp = QLineEdit()
         albumlabel.setBuddy(self._albumdisp)
 
         sortlabel = QLabel(translate("WebDB",
-            'Sort retrieved albums using order:'))
+                                     'Sort retrieved albums using order:'))
         self._sortoptions = QComboBox()
         sortlabel.setBuddy(self._sortoptions)
         editoptions = QPushButton(translate("Defaults", '&Edit'))
-        self.connect(editoptions, SIGNAL('clicked()'), self._editOptions)
-        
+        editoptions.clicked.connect(self._editOptions)
+
         ua_label = QLabel(translate("WebDB",
-            'User-Agent to when accessing web sites.'))
+                                    'User-Agent to when accessing web sites.'))
         self._ua = QTextEdit()
 
         self.jfdi = QCheckBox(translate('Profile Editor',
-            'Brute force unmatched files.'))
+                                        'Brute force unmatched files.'))
         self.jfdi.setToolTip(translate('Profile Editor',
-            "<p>If a proper match isn't found for a file, the files "
-            "will get sorted by filename, the retrieved tag sources "
-            "by filename and corresponding (unmatched) tracks will "
-            "matched.</p>"))
+                                       "<p>If a proper match isn't found for a file, the files "
+                                       "will get sorted by filename, the retrieved tag sources "
+                                       "by filename and corresponding (unmatched) tracks will "
+                                       "matched.</p>"))
 
-        self.matchFields = QLineEdit(u'artist, title')
+        self.matchFields = QLineEdit('artist, title')
         self.matchFields.setToolTip(translate('Profile Editor',
-            "<p>The fields listed here will be used in determining "
-            "whether a track matches the retrieved track. Each "
-            "field will be compared using a fuzzy matching algorithm. "
-            "If the resulting average match percentage is greater "
-            'than the "Minimum Percentage" it\'ll be considered to '
-            "match.</p>"))
+                                              "<p>The fields listed here will be used in determining "
+                                              "whether a track matches the retrieved track. Each "
+                                              "field will be compared using a fuzzy matching algorithm. "
+                                              "If the resulting average match percentage is greater "
+                                              'than the "Minimum Percentage" it\'ll be considered to '
+                                              "match.</p>"))
 
         self.albumBound = QSpinBox()
         self.albumBound.setToolTip(translate('Profile Editor',
-            "<p>The artist and album fields will be used in "
-            "determining whether an album matches the retrieved one. "
-            "Each field will be compared using a fuzzy matching "
-            "algorithm. If the resulting average match percentage "
-            "is greater or equal than what you specify here "
-            "it'll be considered to match.</p>"))
+                                             "<p>The artist and album fields will be used in "
+                                             "determining whether an album matches the retrieved one. "
+                                             "Each field will be compared using a fuzzy matching "
+                                             "algorithm. If the resulting average match percentage "
+                                             "is greater or equal than what you specify here "
+                                             "it'll be considered to match.</p>"))
 
-        self.albumBound.setRange(0,100)
+        self.albumBound.setRange(0, 100)
         self.albumBound.setValue(70)
-        
+
         self.trackBound = QSpinBox()
-        self.trackBound.setRange(0,100)
+        self.trackBound.setRange(0, 100)
         self.trackBound.setValue(80)
-        
+
         vbox = QVBoxLayout()
         vbox.addWidget(label)
         vbox.addWidget(self._text)
-        
+
         vbox.addWidget(albumlabel)
         vbox.addWidget(self._albumdisp)
 
@@ -419,13 +431,13 @@ class SettingsDialog(QWidget):
         frame.setLayout(auto_box)
 
         auto_box.addLayout(create_buddy(translate('Profile Editor',
-                'Minimum &percentage required for album matches.'),
-            self.albumBound))
+                                                  'Minimum &percentage required for album matches.'),
+                                        self.albumBound))
         auto_box.addLayout(create_buddy(translate('Profile Editor',
-            'Match tracks using &fields: '), self.matchFields))
+                                                  'Match tracks using &fields: '), self.matchFields))
         auto_box.addLayout(create_buddy(translate('Profile Editor',
-                'Minimum percentage required for track match.'),
-            self.trackBound))
+                                                  'Minimum percentage required for track match.'),
+                                        self.trackBound))
         auto_box.addWidget(self.jfdi)
 
         vbox.addWidget(frame)
@@ -436,31 +448,31 @@ class SettingsDialog(QWidget):
 
     def applySettings(self, control):
         listbox = control.listbox
-        text = unicode(self._text.text())
+        text = str(self._text.text())
         listbox.trackPattern = text
 
-        albumdisp = unicode(self._albumdisp.text())
+        albumdisp = str(self._albumdisp.text())
         listbox.albumPattern = albumdisp
 
         sort_combo = self._sortoptions
-        sort_options_text = [unicode(sort_combo.itemText(i)) for i in 
-            range(sort_combo.count())]
+        sort_options_text = [str(sort_combo.itemText(i)) for i in
+                             range(sort_combo.count())]
         sort_options = split_strip(sort_options_text)
         listbox.setSortOptions(sort_options)
 
         listbox.sort(sort_options[sort_combo.currentIndex()])
-        
-        useragent = unicode(self._ua.toPlainText())
+
+        useragent = str(self._ua.toPlainText())
         set_useragent(useragent)
 
         listbox.jfdi = self.jfdi.isChecked()
         listbox.matchFields = [z.strip() for z in
-            unicode(self.matchFields.text()).split(u',')]
+                               str(self.matchFields.text()).split(',')]
         listbox.albumBound = self.albumBound.value() / 100.0
         listbox.trackBound = self.trackBound.value() / 100.0
-        
+
         cparser = PuddleConfig(os.path.join(CONFIGDIR, 'tagsources.conf'))
-        set_value = lambda s,v: cparser.set('tagsources', s, v)
+        set_value = lambda s, v: cparser.set('tagsources', s, v)
         set_value('trackpattern', text)
         set_value('albumpattern', albumdisp)
         set_value('sortoptions', sort_options_text)
@@ -474,21 +486,21 @@ class SettingsDialog(QWidget):
         cparser = PuddleConfig(os.path.join(CONFIGDIR, 'tagsources.conf'))
 
         trackpattern = cparser.get('tagsources', 'trackpattern',
-            '%track% - %title%')
+                                   '%track% - %title%')
 
         self._text.setText(trackpattern)
 
         sortoptions = cparser.get('tagsources', 'sortoptions',
-            [u'artist, album', u'album, artist'])
+                                  [u'artist, album', 'album, artist'])
         self._sortoptions.clear()
         self._sortoptions.addItems(sortoptions)
 
         albumformat = cparser.get('tagsources', 'albumpattern',
-            u'%artist% - %album% $if(%__numtracks%, [%__numtracks%], "")')
+                                  '%artist% - %album% $if(%__numtracks%, [%__numtracks%], "")')
         self._albumdisp.setText(albumformat)
 
         self._ua.setText(cparser.get('tagsources',
-            'useragent', 'puddletag/' + puddlestuff.version_string))
+                                     'useragent', 'puddletag/' + version_string))
 
         self.albumBound.setValue(
             cparser.get('tagsources', 'album_bound', 70, True))
@@ -498,18 +510,18 @@ class SettingsDialog(QWidget):
             bool(cparser.get('tagsources', 'jfdi', True, True)))
 
         fields = cparser.get('tagsources', 'match_fields',
-            ['artist', 'title'])
-        fields = u', '.join(z.strip() for z in fields)
+                             ['artist', 'title'])
+        fields = ', '.join(z.strip() for z in fields)
         self.matchFields.setText(fields)
-    
+
     def _editOptions(self):
         text = self._sortoptions.itemText
-        win = SortOptionEditor([text(i) for i in 
-            range(self._sortoptions.count())], self)
-        self.connect(win, SIGNAL('options'), self._setSortOptions)
+        win = SortOptionEditor([text(i) for i in
+                                range(self._sortoptions.count())], self)
+        win.options.connect(self._setSortOptions)
         win.setModal(True)
         win.show()
-    
+
     def _setSortOptions(self, items):
         current = self._sortoptions.currentText()
         self._sortoptions.clear()
@@ -519,6 +531,7 @@ class SettingsDialog(QWidget):
             index = 0
         self._sortoptions.setCurrentIndex(index)
 
+
 def load_source_prefs(name, preferences):
     cparser = PuddleConfig(TAGSOURCE_CONFIG)
     ret = []
@@ -531,6 +544,7 @@ def load_source_prefs(name, preferences):
             ret.append(cparser.get(name, option[0], option[2]))
     return ret
 
+
 def tag_source_search(ts, group, files):
     """Helper method for tag source searches."""
 
@@ -550,23 +564,31 @@ def tag_source_search(ts, group, files):
         if changed:
             audio['album'] = audio['album'].strip()
             write_log(translate('WebDB', 'Retrying search with %s') %
-                audio['album'])
+                      audio['album'])
             ret.extend(ts.search(audio['album'], group[primary]))
 
     return ret, files
 
+
 class MainWin(QWidget):
-    def __init__(self, status, parent = None):
+    writepreview = pyqtSignal(name='writepreview')
+    setpreview = pyqtSignal(object, name='setpreview')
+    clearpreview = pyqtSignal(name='clearpreview')
+    enable_preview_mode = pyqtSignal(name='enable_preview_mode')
+    logappend = pyqtSignal(str, name='logappend')
+    disable_preview_mode = pyqtSignal(name='disable_preview_mode')
+
+    def __init__(self, status, parent=None):
         QWidget.__init__(self, parent)
         self.settingsdialog = SettingsDialog
 
-        connect = lambda obj, sig, slot: self.connect(obj, SIGNAL(sig), slot)
+        connect = lambda obj, sig, slot: getattr(obj, sig).connect(slot)
 
         self.setWindowTitle("Tag Sources")
 
         self.receives = []
         self.emits = ['writepreview', 'setpreview', 'clearpreview',
-            'enable_preview_mode', 'logappend', 'disable_preview_mode']
+                      'enable_preview_mode', 'logappend', 'disable_preview_mode']
 
         self.fieldMapping = audioinfo.mapping
 
@@ -588,7 +610,7 @@ class MainWin(QWidget):
 
         self.sourcelist = QComboBox()
         self.sourcelist.addItems([ts.name for ts in self.__sources])
-        connect(self.sourcelist, 'currentIndexChanged (int)', self.changeSource)
+        connect(self.sourcelist, 'currentIndexChanged', self.changeSource)
 
         sourcelabel = QLabel(translate("WebDB", 'Sour&ce: '))
         sourcelabel.setBuddy(self.sourcelist)
@@ -597,41 +619,41 @@ class MainWin(QWidget):
         preferences.setIcon(QIcon(':/preferences.png'))
         preferences.setToolTip(translate("WebDB", 'Configure'))
         self.__preferencesButton = preferences
-        connect(preferences, 'clicked()', self.configure)
+        connect(preferences, 'clicked', self.configure)
 
         self.searchEdit = QLineEdit()
         self.searchEdit.setToolTip(DEFAULT_SEARCH_TIP)
-        connect(self.searchEdit, 'returnPressed()', self.search)
+        connect(self.searchEdit, 'returnPressed', self.search)
 
         self.searchButton = QPushButton(translate("WebDB", "&Search"))
         self.searchButton.setDefault(True)
         self.searchButton.setAutoDefault(True)
-        connect(self.searchButton, "clicked()", self.search)
+        connect(self.searchButton, "clicked", self.search)
 
         self.submitButton = QPushButton(translate("WebDB",
-            "S&ubmit Tags"))
-        connect(self.submitButton, "clicked()", self.submit)
+                                                  "S&ubmit Tags"))
+        connect(self.submitButton, "clicked", self.submit)
 
         write_preview = QPushButton(translate("WebDB", '&Write'))
-        connect(write_preview, "clicked()", self.writePreview)
-        
+        connect(write_preview, "clicked", self.writePreview)
+
         clear = QPushButton(translate("Previews", "Clea&r preview"))
-        connect(clear, "clicked()",
-            lambda: self.emit(SIGNAL('disable_preview_mode')))
+        connect(clear, "clicked",
+                lambda: self.disable_preview_mode.emit())
 
         self.label = QLabel(translate("WebDB",
-            "Select files and click on Search to retrieve metadata."))
+                                      "Select files and click on Search to retrieve metadata."))
         connect(status_obj, 'statusChanged', self.label.setText)
 
         self.listbox = ReleaseWidget(status, self.curSource)
         self.__updateEmpty = QCheckBox(translate("WebDB",
-            'Update empty fields only.'))
+                                                 'Update empty fields only.'))
         connect(self.listbox, 'statusChanged', self.label.setText)
         connect(self.listbox, 'preview', self.emit_preview)
         connect(self.listbox, 'exactMatches', self.emitExact)
 
         self.__autoRetrieve = QCheckBox(translate("WebDB",
-            'Automatically retrieve matches.'))
+                                                  'Automatically retrieve matches.'))
 
         self.__fieldsEdit = FieldsEdit()
         self.__fieldsEdit.setToolTip(FIELDLIST_TIP)
@@ -641,7 +663,7 @@ class MainWin(QWidget):
         infolabel.setOpenExternalLinks(True)
         connect(self.listbox, 'infoChanged', infolabel.setText)
 
-        connect(status_obj, 'logappend', SIGNAL('logappend'))
+        connect(status_obj, 'logappend', self.logappend)
 
         sourcebox = QHBoxLayout()
         sourcebox.addWidget(sourcelabel)
@@ -671,7 +693,7 @@ class MainWin(QWidget):
         vbox.addWidget(self.__autoRetrieve)
         self.setLayout(vbox)
         self.changeSource(0)
-        
+
     def _applyPrefs(self, prefs):
         self.curSource.applyPrefs(prefs)
         cparser = PuddleConfig(TAGSOURCE_CONFIG)
@@ -699,7 +721,6 @@ class MainWin(QWidget):
         self.searchEdit.setEnabled(hasattr(self.curSource, 'keyword_search'))
 
         self.submitButton.setVisible(hasattr(self.curSource, 'submit'))
-            
 
     def configure(self):
         config = getattr(self.curSource, 'preferences', None)
@@ -723,13 +744,13 @@ class MainWin(QWidget):
                     pref[2] = value
             win = SimpleDialog(self.curSource.name, prefs, self)
         win.setModal(True)
-        self.connect(win, SIGNAL('editingFinished'), self._applyPrefs)
+        win.editingFinished.connect(self._applyPrefs)
         win.show()
-    
+
     def emit_preview(self, tags):
         if not self.__updateEmpty.isChecked():
-            self.emit(SIGNAL('enable_preview_mode'))
-            self.emit(SIGNAL('setpreview'), tags)
+            self.enable_preview_mode.emit()
+            self.setpreview.emit(tags)
         else:
             files = self._status['selectedfiles']
             previews = []
@@ -739,13 +760,13 @@ class MainWin(QWidget):
                     if field not in f:
                         temp[field] = r[field]
                 previews.append(temp)
-            self.emit(SIGNAL('enable_preview_mode'))
-            self.emit(SIGNAL('setpreview'), previews)
+            self.enable_preview_mode.emit()
+            self.setpreview.emit(previews)
 
     def emitExact(self, d):
         if not self.__updateEmpty.isChecked():
-            self.emit(SIGNAL('enable_preview_mode'))
-            self.emit(SIGNAL('setpreview'), d)
+            self.enable_preview_mode.emit()
+            self.setpreview.emit(d)
         else:
             previews = []
             for f, r in d.items():
@@ -754,8 +775,8 @@ class MainWin(QWidget):
                     if field not in f:
                         temp[field] = r[field]
                 previews.append(temp)
-            self.emit(SIGNAL('enable_preview_mode'))
-            self.emit(SIGNAL('setpreview'), previews)
+            self.enable_preview_mode.emit()
+            self.setpreview.emit(previews)
 
     def loadSettings(self):
         settings = PuddleConfig(os.path.join(CONFIGDIR, 'tagsources.conf'))
@@ -763,22 +784,22 @@ class MainWin(QWidget):
 
         source = get('lastsource', 'Musicbrainz')
         self.__sourceFields = [settings.get('tagsourcetags', ts.name, [])
-            for ts in self.__sources]
+                               for ts in self.__sources]
 
         index = self.sourcelist.findText(source)
         if index == -1:
             index = 0
         self.sourcelist.setCurrentIndex(index)
         self.__fieldsEdit.setTags(self.__sourceFields[index])
-        df = get('trackpattern', u'%track% - %title%')
+        df = get('trackpattern', '%track% - %title%')
         self.listbox.trackPattern = df
 
         albumformat = get('albumpattern',
-            u'%artist% - %album%$if(%__numtracks%, [%__numtracks%], "")')
+                          '%artist% - %album%$if(%__numtracks%, [%__numtracks%], "")')
         self.listbox.albumPattern = albumformat
 
         sort_options = get('sortoptions',
-            [u'artist, album', u'album, artist'])
+                           [u'artist, album', 'album, artist'])
         sort_options = split_strip(sort_options)
         self.listbox.setSortOptions(sort_options)
 
@@ -805,21 +826,21 @@ class MainWin(QWidget):
 
     def setResults(self, retval):
         self.searchButton.setEnabled(True)
-        if isinstance(retval, (basestring, QString)):
+        if isinstance(retval, (str, str, str)):
             self.label.setText(retval)
         else:
             releases, files = retval
             if releases:
                 self.label.setText(translate("WebDB",
-                    'Searching complete.'))
+                                             'Searching complete.'))
             else:
                 self.label.setText(translate("WebDB",
-                    'No matching albums were found.'))
+                                             'No matching albums were found.'))
             if files and self.__autoRetrieve.isChecked():
                 self.listbox.setReleases(releases, files)
             else:
                 self.listbox.setReleases(releases)
-            self.listbox.emit(SIGNAL('infoChanged'), '')
+            self.listbox.infoChanged.emit('')
 
     def search(self):
         if not self.searchButton.isEnabled():
@@ -830,10 +851,10 @@ class MainWin(QWidget):
         self.label.setText(translate("WebDB", 'Searching...'))
         text = None
         if self.searchEdit.text() and self.searchEdit.isEnabled():
-            text = unicode(self.searchEdit.text())
+            text = str(self.searchEdit.text())
         elif not files:
             self.label.setText(translate("WebDB",
-                '<b>Select some files or enter search paramaters.</b>'))
+                                         '<b>Select some files or enter search paramaters.</b>'))
             return
 
         def search():
@@ -843,16 +864,17 @@ class MainWin(QWidget):
                     return self.curSource.keyword_search(text), None
                 else:
                     return tag_source_search(self.curSource, group, files)
-            except RetrievalError, e:
+            except RetrievalError as e:
                 return translate('WebDB',
-                    'An error occured: %1').arg(unicode(e))
-            except Exception, e:
+                                 'An error occured: %1').arg(str(e))
+            except Exception as e:
                 traceback.print_exc()
                 return translate('WebDB',
-                    'An unhandled error occurred: %1').arg(unicode(e))
+                                 'An unhandled error occurred: %1').arg(str(e))
+
         self.searchButton.setEnabled(False)
         t = PuddleThread(search, self)
-        self.connect(t, SIGNAL('threadfinished'), self.setResults)
+        t.threadfinished.connect(self.setResults)
         t.start()
 
     def submit(self):
@@ -866,19 +888,19 @@ class MainWin(QWidget):
         def submit():
             try:
                 self.curSource.submit(files)
-            except SubmissionError, e:
+            except SubmissionError as e:
                 traceback.print_exc()
                 return translate('WebDB',
-                    'An error occured: %1').arg(unicode(e))
-            except Exception, e:
+                                 'An error occured: %1').arg(str(e))
+            except Exception as e:
                 traceback.print_exc()
                 return translate('WebDB',
-                    'An unhandled error occurred: %1').arg(unicode(e))
+                                 'An unhandled error occurred: %1').arg(str(e))
 
             return translate("WebDB", "Submission completed.")
 
         t = PuddleThread(submit, self)
-        self.connect(t, SIGNAL('threadfinished'), end)
+        t.threadfinished.connect(end)
         t.start()
 
     def saveSettings(self):
@@ -890,8 +912,8 @@ class MainWin(QWidget):
         settings.set('tagsources', 'lastsort', self.listbox.lastSortIndex)
         settings.set('tagsources', 'existing', self.__updateEmpty.isChecked())
         settings.set('tagsources', 'autoretrieve',
-            self.__autoRetrieve.isChecked())
-    
+                     self.__autoRetrieve.isChecked())
+
     def setMapping(self, mapping):
         self.fieldMapping = mapping
         if self.curSource.name in mapping:
@@ -900,9 +922,10 @@ class MainWin(QWidget):
             self.listbox.setMapping({})
 
     def writePreview(self):
-        self.emit(SIGNAL('writepreview'))
+        self.writepreview.emit()
         self.label.setText(translate("WebDB", "<b>Tags were written.</b>"))
 
+
 control = ('Tag Sources', MainWin, RIGHTDOCK, False)
 
 if __name__ == '__main__':
@@ -911,4 +934,4 @@ if __name__ == '__main__':
     status['selectedfiles'] = exampletags.tags
     win = MainWin(status)
     win.show()
-    app.exec_()
\ No newline at end of file
+    app.exec_()
